/*
 * (C) Copyright Pattern Recognition Group Delft. All rights reserved.
 * 
 * MATINV : Matrix inversion program
 * 
 * References : Press W, Flannery B, Teukolsky S, Vetterling W, "Numerical
 * Recipes in C", Cambridge University Press, 1988. See chapter 2.
 * 
 * Made by : W.F. Schmidt 
 * SccsId  : %Z%$RCSfile: matinv.c,v $ V%Q% $Revision: 2.4 $ $Date: 1996/10/07 06:50:00 $
 */

#define MATINV
#define SPRLIB
#include <sprlib.h>

/*
 * MATINV : Calculate the inverse matrix of a[1..n][1..n]. Matrix a is not
 * destroyed by this routine, which is not standard in NR routines. This
 * routine is copied from page 45 of Numerical Recipes in C and adapted to
 * produce the inverse matrix without destroying the input. The programming
 * is in similar style as in NR in C.
 *
 * Return value : TRUE  If LUDCMP reports singular matrix.
 *                FALSE If no error detected.
 */

int matinv (double **a, int n, double **inv)
{
   int i, j, *indx;
   double **lu, *col, d;
   int *ivector();
   double **matrix(), *vector();
   void free_ivector(), free_vector(), free_matrix();
   int ludcmp();
	void lubksb();

   col = vector(1, n);
   indx = ivector(1, n);
   lu = matrix(1, n, 1, n);

   for (i = 1; i <= n; i++)
   {
      for (j = 1; j <= n; j++)
         lu[i][j] = a[i][j];
   }
   if (ludcmp(lu, n, indx, &d))
		return TRUE;

   for (j = 1; j <= n; j++)
   {
      for (i = 1; i <= n; i++)
         col[i] = 0.0;
      col[j] = 1.0;
      lubksb(lu, n, indx, col);
      for (i = 1; i <= n; i++)
         inv[i][j] = col[i];
   }

   free_vector(col, 1, n);
   free_ivector(indx, 1, n);
   free_matrix(lu, 1, n, 1, n);

	return FALSE;
}

/*
 * MATINVD : Calculate the inverse matrix of a[1..n][1..n]. Matrix a is not
 * destroyed by this routine, which is not standard in NR routines. This
 * routine is copied from page 45 of Numerical Recipes in C and adapted to
 * produce the inverse matrix without destroying the input. The programming
 * is in similar style as in NR in C.
 *
 * This version also returns the determinant value of the inverse matrix.
 *
 * Return value : TRUE  If LUDCMP reports singular matrix.
 *                FALSE If no error detected.
 */

int matinvd (double **a, int n, double **inv, double *det)
{
   int i, j, *indx;
   double **lu, *col, d;
   int *ivector();
   double **matrix(), *vector();
   void free_ivector(), free_vector(), free_matrix();
   int ludcmp();
	void lubksb();

   col = vector(1, n);
   indx = ivector(1, n);
   lu = matrix(1, n, 1, n);

   for (i = 1; i <= n; i++)
   {
      for (j = 1; j <= n; j++)
         lu[i][j] = a[i][j];
   }
   if (ludcmp(lu, n, indx, &d))
		return TRUE;

	*det = lu[1][1];
	for (i=2; i<=n; i++)
		*det *= lu[i][i];
	*det = 1.0 / (*det);

   for (j = 1; j <= n; j++)
   {
      for (i = 1; i <= n; i++)
	 col[i] = 0.0;
      col[j] = 1.0;
      lubksb(lu, n, indx, col);
      for (i = 1; i <= n; i++)
         inv[i][j] = col[i];
   }

   free_vector(col, 1, n);
   free_ivector(indx, 1, n);
   free_matrix(lu, 1, n, 1, n);

	return FALSE;
}

#undef MATINV
#undef SPRLIB

/*
 * End of Source
 */
